@@ -6,6 +6,11 @@ import paramiko
 import subprocess
 import os
 import sys
+import tarfile
+import random
+import string
+import tempfile
+import shutil
 
 from hwinfo.pci import PCIDevice
 from hwinfo.pci.lspci import *
@@ -40,6 +45,62 @@ def local_command(cmd):
         print stdout
         raise Exception("stderr: %s" % str(stderr))
 
+def find_in_tarball(tarball, filename):
+    tar = tarfile.open(tarball)
+    members = tar.getmembers()
+    tar.close()
+    matches = []
+    for m in members:
+        if m.name.endswith(filename):
+            matches.append(m.name)
+
+    if len(matches) > 1:
+        raise Exception("Error: more than one match for that filename")
+
+    if not matches:
+        raise FileNotFound("Cound not find %s in %s" % (filename, tarball))
+    return matches.pop()
+
+def read_files_from_tarball(tarball, files):
+    tar = tarfile.open(tarball)
+    data = None
+    tmpdir = tempfile.mkdtemp()
+
+
+    file_data = {}
+    for f in files:
+        tar.extract(f, tmpdir)
+        file_data[os.path.basename(f)] = read_from_file("%s/%s" % (tmpdir, f))
+
+    tar.close()
+    if os.path.exists(tmpdir):
+        shutil.rmtree(tmpdir)
+    return file_data
+
+def read_from_tarball(tarball, filename):
+    tar = tarfile.open(tarball)
+    data = None
+    tmpdir = tempfile.mkdtemp()
+
+    tar.extract(filename, tmpdir)
+    data = read_from_file("%s/%s" % (tmpdir, filename))
+
+    tar.close()
+    if os.path.exists(tmpdir):
+        shutil.rmtree(tmpdir)
+
+    return data
+
+def parse_kvp_string(data):
+    rec = {}
+    lines = data.split('\n')
+    for line in lines:
+        if not line:
+            continue
+        k, v = line.split('=')
+        rec[k] = v.strip("'")
+    return rec
+
 class Host(object):
 
     client = None
@@ -73,6 +134,17 @@ class Host(object):
     def get_cpuinfo_data(self):
         return self.exec_command(['cat /proc/cpuinfo'])
 
+    def get_os_data(self):
+         return self.exec_command(['cat', '/etc/xensource-inventory'])
+
+    def get_os_info(self):
+        rec = {}
+        os_rec = parse_kvp_string(self.get_os_data())
+        rec['os'] = os_rec['PRODUCT_BRAND']
+        rec['version'] = os_rec['PRODUCT_VERSION']
+        rec['build'] = os_rec['BUILD_NUMBER']
+        return rec
+
     def get_pci_devices(self):
         data = self.get_lspci_data()
         parser = LspciNNMMParser(data)
@@ -83,6 +155,14 @@ class Host(object):
         data = self.get_dmidecode_data()
         parser = dmidecode.DmidecodeParser(data)
         rec = parser.parse()
+        try:
+            os_rec = self.get_os_info()
+            for k, v in os_rec.iteritems():
+                rec[k] = v
+        except:
+            #Ignore failures. Only supports XS right now.
+            pass
+
         return rec
 
     def get_cpu_info(self):
@@ -142,6 +222,9 @@ class HostFromLogs(Host):
     def get_cpuinfo_data(self):
         return self._load_from_file('cpuinfo')
 
+    def get_os_data(self):
+        return self._load_from_file('xensource-inventory')
+
     def get_pci_devices(self):
         try:
             devs = super(HostFromLogs, self).get_pci_devices()
@@ -155,6 +238,38 @@ class HostFromLogs(Host):
             recs = combine_recs(all_recs, 'pci_device_bus_id')
             return [PCIDevice(rec) for rec in recs]
 
+class HostFromTarball(HostFromLogs):
+
+    def __init__(self, filename):
+        self.tarloc = filename
+        pre_load_files = [
+            'lspci-nnm.out',
+            'lspci-vv.out',
+            'lspci-n.out',
+            'dmidecode.out',
+            'cpuinfo',
+            'xensource-inventory'
+        ]
+
+        self._preload_files(pre_load_files)
+
+    def _preload_files(self, filenames):
+        paths = []
+        for f in filenames:
+            try:
+                filepath = find_in_tarball(self.tarloc, f)
+                paths.append(filepath)
+            except FileNotFound:
+                continue
+        self.fdata = read_files_from_tarball(self.tarloc, paths)
+
+    def _load_from_file(self, filename):
+        """Find filename in tar, and load it"""
+        if filename in self.fdata:
+            return self.fdata[filename]
+        else:
+            filepath = find_in_tarball(self.tarloc, filename)
+            return read_from_tarball(self.tarloc, filepath)
 
 def pci_filter(devices, types):
     res = []
